/***************************************************************************
                               cmudstream.h
                             -------------------
    begin                : Fri Aug 10 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMUDSTREAM_H
#define CMUDSTREAM_H

#include <qobject.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qstringlist.h>

class KmudFilter;



/**
  * This class is the central place where all input to/output from the MUD
  * goes through. Here you can register a @ref KmudFilter to access this
  * data.
  *@author Kmud Developer Team
  */

class CMudStream : public QObject
{
	Q_OBJECT

public:
	/*+ constructor */
	CMudStream(QObject* parent=0, const char* name=0);
	/** destructor */
	~CMudStream();

	/**
	 * Registers a @ref KmudFilter as an input filter.
	 * You can define the position of the filter in the list relative to other filters.
	 *
	 * @param after  your filter will be registered after all filters which names contain one of the strings in this stringlist.
	 * @param before  your filter will be registered before all filters which names contain one of the strings in this stringlist.
	 *
	 * The strings in after and before are "fuzzy", so e.g. before = "mapper" will place your filter before _all_ filters
	 * which name contain the string "mapper".
	 *
	 * It is possible to register the same filter more than once.
	 */
	bool registerInputFilter(const KmudFilter* f, const QStringList after=QStringList(), const QStringList before=QStringList("All"));

	/**
	 * Registers a @ref KmudFilter as an output filter.
	 *
	 * See @ref registerInputFilter for more information about after and before.
	 */
	bool registerOutputFilter(const KmudFilter* f, const QStringList after=QStringList(), const QStringList before=QStringList("All"));

	/** Removes all occurences of a @ref KmudFilter from the input filter list. */
	bool removeInputFilter(const KmudFilter* f);
	/** Removes all occurences of a @ref KmudFilter from the output filter list. */
	bool removeOutputFilter(const KmudFilter* f);


	/** Returns a list of all input filter names which are registered before a certain KmudFilter */
	QStringList inputFiltersBefore(const KmudFilter* f);
	/** Returns a list of all input filter names which are registered after a certain KmudFilter */
	QStringList inputFiltersAfter(const KmudFilter* f);
	/** Returns a list of all output filter names which are registered before a certain KmudFilter */
	QStringList outputFiltersBefore(const KmudFilter* f);
	/** Returns a list of all output filter names which are registered after a certain KmudFilter */
	QStringList outputFiltersAfter(const KmudFilter* f);


public slots:
	/** processes input to the mud */
	void processInput(QString);
	/** processes output from the mud */
	void processOutput(QString);

signals:
	/** emitted when input was processed */
	void inputProcessed();
	/** emitted when output was processed */
	void outputProcessed();

private:

	/** Calculates the desired list position for inserting a filter */
	int calculateFilterPosition(const QPtrList<KmudFilter>, const QStringList after, const QStringList before);


	QPtrList<KmudFilter> m_inputFilterList;
	QPtrList<KmudFilter> m_outputFilterList;
};

#endif

