/***************************************************************************
                               kmudapi.h
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUDAPI_H
#define KMUDAPI_H

// Qt includes
#include <qobject.h>
#include <qstringlist.h>

class KConfig;
class KmudViewManager;
class CMudStream;
class KStatusBar;
class QDomElement;

/**
  *@author Kmud Developer Team
  */


class ConfigAPI : public QObject
{
	Q_OBJECT

public:
	virtual KConfig* kmudConfig() = 0;

	/**
	  * Use this method to notify others about changes to the global kmud config.
	  * @param group the KConfig group the changed keys belonged to.
	  */
	inline void emitConfigGroupChanged(const QString& group) {
		if (!group.isEmpty())
			emit configGroupChanged(group);
	}

signals:
	/**
	  * Connect this to an appropriate slot to get notified about data changes.
	  */
	void configGroupChanged(const QString& group);
};

class ProfileAPI : public QObject
{
	Q_OBJECT

public:
	/**
	  * Types of data stored in a profile.
	  */
	static const uint ALL     = 7;
	static const uint CONFIG  = 1;
	static const uint ALIAS   = 2;
	static const uint TRIGGER = 4;
	
	/**
	  * Open a character and/or mud profile.
	  * If create is true the profile(s) will be created if loading fails.
	  * If the loading fails and create was false, the returned DOM element is null
	  * (QDomElement.isNull() == true).
	  * The returned DOM element is the <profile> Element.
	  * One of its child elements is a <config> element named "/" (the root config element).
	  *
	  * Example:
	  * openProfile("MyMud"); will try to open the mud profile of MyMud.
	  * openProfile("MyMud/MyChar"); will tro to open mud profile MyMud and character profile MyChar.
	  */
	virtual QDomElement openProfile(const QString& profile, bool create = false) = 0;
	/**
	  * Saves the specified profile.
	  * If the profile doesn't exist or the writing the profile data failed it will return false.
	  */
	virtual bool saveProfile(const QString& profile) = 0;

	/**
	  * Retrieves a list of profile belonging to a base profile.
	  * If the baseProfile string is empty it will return a list of mud profiles
	  * If it contains the name of a mud profile, it will return a list of character
      * profiles of this mud.
	  */
	virtual QStringList listProfiles(const QString& baseProfile = QString::null) const = 0;

	/**
	  * Retrieves a list of directories where data belonging to a specified
	  * mud profile could be located.
	  * The directories do not have to contain the profile itself.
	  */
	virtual QStringList listProfileDirs(const QString& mudProfile) const = 0;

	/**
	  * Returns the directory to which data related to the specified mud profile
	  * should be saved.
	  * This is the directory the profile itself will be saved to.
	  * If the directory does not exist yet, it will be created.
	  */
	virtual QString defaultSavePath(const QString& mudProfile) = 0;

	/**
	  * Use this method to notify any other profile user about a data change.
	  * @param profile the profile of which data got changed
	  * @param typeOfData the categorie of data that got changed.
	  */
	inline void emitDataChanged(const QString& profile, uint typeOfData = ALL) {
		if (!profile.isEmpty())
			emit profileDataChanged(profile, typeOfData);
	}

signals:
	/**
	  * Connect this to an appropriate slot to get notified about data changes.
	  */
	void profileDataChanged(const QString& profile, uint typeOfData);
};

class KmudAPI : public QObject
{
	Q_OBJECT

public:
	/**
	  * possible errors which can occur during a connection attempt
	  */
	enum ConnectionError
	{
		ErrConnectionRefused,
		ErrHostNotFound,
		ErrTimeout,
		ErrUnknown
	};

	/**
	  * possible states of the MUD connection
	  */
	enum ConnectionState
	{
		Disconnected,
		Connecting,
		Connected
	};

	KmudAPI();
	virtual ~KmudAPI();
	
	virtual KmudViewManager* viewManager()=0;
	virtual CMudStream* mudStream() = 0;
	
    virtual ConfigAPI* configAPI() = 0;
	
	virtual ProfileAPI* profileAPI() = 0;
	
	/**
	  * Returns the currently active profile or QString::null.
	  */
	virtual QString profile() = 0;

	/**
	  * if there was an error during the last connection attempt, then
	  * this method will give you the kind of error
	  */
	virtual ConnectionError connectionError()=0;

	/**
	  * returns the current state of the MUD connection
	  */
	virtual ConnectionState connectionState()=0;

	/**
	  * appends text to the output view
	  */
	virtual void appendOutput(QString)=0;

	/**
	 * This is used to send a command to the mud
	 */
	virtual void sendCommand(QString,bool)=0;

	/**
	  * returns a pointer to the statusbar of the mainwindow
	  */
	virtual KStatusBar* mainStatusBar()=0;

signals:
	/**
	  * emitted when the connection state changed
	  * you can then get the current state with connectionState()
	  */
	void connectionStateChanged();
	
	/** emitted when the current profile was changed */
	void profileChanged();
	
private:

};

#endif

